import { Injectable, Logger, InternalServerErrorException } from '@nestjs/common';
import { ContactForm, Email, EmailWithoutFrom, FeedbackForm } from './email';
import { ConfigService } from '../configuration/config.service';
import { buildContactAdminEmail, buildContactUserEmail } from '../email-templates/contact';
import { buildFeedbackEmail } from '../email-templates/feedback';
import * as useragent from 'useragent';
import * as Nodemailer from 'nodemailer';
import moment = require('moment-timezone');
moment.tz.setDefault('Europe/Paris');

@Injectable()
export class EmailService {

  private logger: Logger;
  private config;

  constructor(private configService: ConfigService) {
    this.logger = new Logger(EmailService.name);
    this.config = this.configService.config;
  }

  async sendContactEmails(contactForm: ContactForm) {
    this.logger.log('Entering function', `${EmailService.name} - ${this.sendContactEmails.name}`);

    const adminEmailBody = buildContactAdminEmail({
      subject: contactForm.subject,
      message: contactForm.text,
      firstName: contactForm.firstname,
      lastName: contactForm.lastname,
      email: contactForm.email,
      datetime: moment().format('DD/MM/YYYY à HH:mm'),
      imageHost: this.config.imageHost,
    });
    const userEmailBody = buildContactUserEmail({
      subject: contactForm.subject,
      message: contactForm.text,
      firstName: contactForm.firstname,
      datetime: moment().format('DD/MM/YYYY à HH:mm'),
      imageHost: this.config.imageHost,
    });

    const adminEmail = new EmailWithoutFrom();
    adminEmail.to = [this.config.userSupportMailbox];
    adminEmail.replyTo = contactForm.email;
    adminEmail.subject = contactForm.subject;
    adminEmail.html = adminEmailBody;

    const userEmail = new EmailWithoutFrom();
    userEmail.to = [contactForm.email];
    userEmail.replyTo = this.config.userSupportMailbox;
    userEmail.subject = contactForm.subject;
    userEmail.html = userEmailBody;

    await this.send(adminEmail);
    await this.send(userEmail);

    return;
  }

  async sendFeedback(feedbackForm: FeedbackForm, userAgent) {
    this.logger.log('Entering function', `${EmailService.name} - ${this.sendFeedback.name}`);

    const userAgentParsed = useragent.parse(userAgent);
    let userAgentString = '';

    if (userAgentParsed) {
      userAgentString = `Family: ${userAgentParsed.family},`;
      userAgentString += `Version:${userAgentParsed.major}.${userAgentParsed.minor}.${userAgentParsed.patch},`;
      userAgentString += `Source: ${userAgentParsed.source}`;
    }

    const feelings = [
      'Pas d\'avis',
      'Bof...',
      'Pas mal',
      'Top!'
    ]

    const feeling = feedbackForm.feeling ? feelings[feedbackForm.feeling] : '-';
    const email = feedbackForm.email ? feedbackForm.email : '-';

    const feedbackEmailBody = buildFeedbackEmail({
      feeling,
      email,
      url: feedbackForm.url,
      userAgent: userAgentString,
      version: feedbackForm.version,
      message: feedbackForm.message,
      datetime: moment().format('DD/MM/YYYY à HH:mm'),
      imageHost: this.config.imageHost,
    });

    const feedbackEmail = new EmailWithoutFrom();
    feedbackEmail.to = [this.config.userSupportMailbox];

    if (this.config.additionalFeedbackEmails) {
      feedbackEmail.to = feedbackEmail.to.concat(this.config.additionalFeedbackEmails.split(','));
    }

    if (feedbackForm.email) {
      feedbackEmail.replyTo = feedbackForm.email;
    }

    feedbackEmail.subject = 'Feedback';
    feedbackEmail.html = feedbackEmailBody;

    await this.send(feedbackEmail);

    return;
  }

  async send(emailInfo: EmailWithoutFrom) {
    this.logger.log('Entering function', `${EmailService.name} - ${this.send.name}`);

    let email = new Email();
    email.from = this.config.noReplyMailAddress;
    email = Object.assign(email, emailInfo);

    if (this.config.mailSubjectPrefix) {
      email.subject = `[${this.config.mailSubjectPrefix}] ${email.subject}`;
    }

    const transporter = Nodemailer.createTransport(this.config.smtpConfig);

    await transporter.verify().catch((error) => {
      this.logger.error('SMTP connection failed.', `${error}`, `${EmailService.name} - ${this.send.name}`);
      throw new InternalServerErrorException({ error, message: 'SMTP connection failed.' });
    });

    this.logger.log('SMTP Server is ready to receive messages', `${EmailService.name} - ${this.send.name}`);

    await transporter.sendMail(email).catch((error) => {
      this.logger.error('Couldn\'t send email.', `${error}`, `${EmailService.name} - ${this.send.name}`);
      transporter.close();
      throw new InternalServerErrorException({ error, message: 'Couldn\'t send email.' });
    });

    transporter.close();
    this.logger.log('Email sent to SMTP server', `${EmailService.name} - ${this.send.name}`);

    return;
  }

}
